// importing the required packages from D3 
import {
  select,
  csv,
  scaleLinear,
  scaleTime,
  extent,
  axisLeft,
  axisBottom,
  format,
  line,
  area,
  curveBasis
} from 'd3';
//declaring the variables 
const svg = select('svg');

const width = +svg.attr('width');
const height = +svg.attr('height');

// Formatting the total value to show on the piechart
const xAxisTickFormat = number =>
    format('.3s')(number)

const render = data => {
  
  const xValue = d => d.Year;
  const xAxisLabel = 'Year';
  
  const yValue = d => d.CrimeRate;
  const circleRadius = 7;
  const yAxisLabel = 'CrimeRate';
  
  const margin = { top: 10, right: 40, bottom: 130, left: 150 };
  const innerWidth = width - margin.left - margin.right;
  const innerHeight = height - margin.top - margin.bottom;  
// Determing the Scale for X axis
  const xScale = scaleLinear()
    .domain(extent(data, xValue))
    .range([0, innerWidth])
    .nice();
// Determing the Scale for Y axis  
  const yScale = scaleLinear()
    .domain(extent(data, yValue))
    .range([innerHeight, 0])
    .nice();
 // POsitioning the svg element   
  const g = svg.append('g')
    .attr('transform', `translate(${margin.left},${margin.top})`);
 // Assigning values to x axis 
  const xAxis = axisBottom(xScale)
    .tickSize(-innerHeight)
    .tickPadding(15);
  
 // Assigning values to y axis  
  const yAxis = axisLeft(yScale)
    .tickSize(-innerWidth)
    .tickPadding(10)
    .tickFormat(xAxisTickFormat);
  
  const yAxisG = g.append('g').call(yAxis);
  yAxisG.selectAll('.domain').remove();
  // assigning label for y axis
  yAxisG.append('text')
      .attr('class', 'axis-label')
      .attr('y', -93)
      .attr('x', -innerHeight / 2)
      .attr('transform', `rotate(-90)`)
      .attr('text-anchor', 'middle')
      .text(yAxisLabel);
  
  const xAxisG = g.append('g').call(xAxis)
    .attr('transform', `translate(0,${innerHeight})`);
  //assigning label for x axis
  xAxisG.append('text')
      .attr('class', 'axis-label')
      .attr('y', 75)
      .attr('x', innerWidth / 2)
      .text(xAxisLabel);
 // drawing the line on the curve   and adding it to svg 
  const lineGenerator = line()
  	.x(d => xScale(xValue(d)))
  	.y(d => yScale(yValue(d)))
  	.curve(curveBasis);
 // drawing the area under the curve and adding it to svg 
  const areaGenerator = area()
  	.x(d => xScale(xValue(d)))
  	.y0(innerHeight)
  	.y1(d => yScale(yValue(d)))
  	.curve(curveBasis);
  // generating linechart for the data 
  g.append('path')
  	.attr('class', 'line-chart')
  	.attr('d', lineGenerator(data));
  // generating areachart for the dataset
  g.append('path')
  	.attr('class', 'area-chart')
  	.attr('d', areaGenerator(data));
  
 
};
// Loading the csv data
csv('https://gist.githubusercontent.com/kakumanusreya/a6d0e3556b624d0b9de2fb54771177ad/raw/70eed12c92498845a7f56023db0204e8b74086f9/crimerate.csv')
  .then(data => {
    data.forEach(d => {
      d.CrimeRate = +d.CrimeRate;
//Formatting to retrieve year from the date
      d.Year = +d.Year;
    });
    render(data);
  	console.log(data);
  });